import numpy as np
import matplotlib.pyplot as P
from scipy.optimize import brentq

P.rcParams['font.size'] = '14'
prop_cycle = P.rcParams['axes.prop_cycle']
color = prop_cycle.by_key()['color']

def impli(x,h,a):
    return np.cos(x)*np.sin(x)-h*np.cos(a)*np.cos(x)+h*np.sin(a)*np.sin(x)

def hex_to_RGB(hex_str):
    """ #FFFFFF -> [255,255,255]"""
    return [int(hex_str[i:i+2], 16) for i in range(1,6,2)]

def get_color_gradient(c1, c2, n):
    """
    Given two hex colors, returns a color gradient
    with n colors.
    """
    assert n > 1
    c1_rgb = np.array(hex_to_RGB(c1))/255
    c2_rgb = np.array(hex_to_RGB(c2))/255
    mix_pcts = [x/(n-1) for x in range(n)]
    rgb_colors = [((1-mix)*c1_rgb + (mix*c2_rgb)) for mix in mix_pcts]
    return ["#" + "".join([format(int(round(val*255)), "02x") for val in item]) for item in rgb_colors]
    
# best fit (with double angle correction)
nuhf_a = 60.70 # (6) MHz
enuhf_a = 0.007 # MHz
nuhf_c = 64.23#(3) MHz
enuhf_c =0.05
nuQ = 4.49#(3) MHz
enuQ = 0.03
eta = 0
phi = 5.1 # deg
ephi = 0.6 # deg
p = np.genfromtxt('Data/Cr53_NMRlines_vs_H_extended.dat',skip_header=2) # these are NMR data, H, centers of triplet peaks and errors
H = p[:,0]
nu1 = p[:,1]
enu1 = p[:,2]
nu2 = p[:,3]
enu2 = p[:,4]
nu3 = p[:,5]
enu3 = p[:,6]

p = np.genfromtxt('Data/calculated_line_triplets_tiltedxtal.dat') # notice 1 and 3 are exchanged, these are Giuseppe's best fit values
nu_th3 = p[:,1]
nu_th2 = p[:,2]
nu_th1 = p[:,3]

p = np.genfromtxt('Data/SpinAngles_vs_H_correct4tilt.dat') # corrected, used to calculate cot_th0 to become th0
theta = p[:,1]
etheta = p[:,2]
p = np.genfromtxt('Data/SpinAngles_vs_H_tiltedxtal.dat') # not corrected
theta0 = p[:,1]
etheta0 = p[:,2]

# apply correction twice all angles inrad
alpha, ealpha = 8.8855115e-02, 1.1057429e-02
sa,ca = np.sin(alpha), np.cos(alpha)
cot_th0 = ca/np.tan(np.deg2rad(theta)) - sa
th0 = np.pi/2 - np.arctan(cot_th0)
x=1/(1 + cot_th0**2)
dth0_dth = x*ca/np.sin(np.deg2rad(theta))**2
dth0_da = x*(sa/np.tan(np.deg2rad(theta)) + ca) 
eth0 = np.sqrt((dth0_dth*np.deg2rad(etheta))**2+(dth0_da*ealpha)**2) # in rad
#st,ct, et = np.sin(np.deg2rad(theta0)),np.cos(np.deg2rad(theta0)), np.deg2rad(etheta0)
#thetac = np.rad2deg(np.pi/2-np.arctan(ca*ct - sa))
#ethetac =np.rad2deg(np.sqrt(((sa*ct+ca)*ealpha)**2+(ca*st*etheta0)**2)/(1+(ca*ct-sa)**2))
fig,[ax1,ax2] = P.subplots(2,1,figsize=(5,7),sharex=True)
dx=0.14
dx0=0.125
for k in range(11):
    ax1.fill_between([2+k*dx,2+k*dx+dx0],[65,65],y2=[50,50],color = color[0],alpha=k*0.05)
ax1.errorbar(H,nu1,yerr=enu1,fmt='o')
ax1.errorbar(H,nu2,yerr=enu2,fmt='o')
ax1.errorbar(H,nu3,yerr=enu3,fmt='o')
ax1.plot(H,nu_th1,'-',color=color[0])
ax1.plot(H,nu_th2,'-',color=color[1])
ax1.plot(H,nu_th3,'-',color=color[2])
ax1.set_ylabel(r'$\nu$ [MHz]')
ax1.text(0,57.3,'m=1')
ax1.text(0,59.8,'m=0')
ax1.text(0,63.5,'m=-1')
ax1.text(2.8,59,'FFM')
ax1.text(0.8,54,'CCP')
ax1.text(0,52,'(a)')
for k in range(11):
    ax2.fill_between([2+k*dx,2+k*dx+dx0],[100,100],y2=[-1,-1],color = color[0],alpha=k*0.05)


h = np.linspace(0,1,101)
th = np.zeros(101)
for k in range(101):
    th[k]  = brentq(impli,0,np.pi/2,args=(h[k],alpha))

ax2.errorbar(H,theta0,yerr=etheta0,fmt='s',color='k',mfc='w',alpha=0.3)
#ax2.errorbar(H,np.rad2deg(th0),yerr=np.rad2deg(eth0),fmt='s',color=color[4])
ax2.errorbar(H,theta,yerr=etheta,fmt='s',color=color[4])
#ax2.plot(H,H-H+90,'k-',lw=0.5)
Hc = 1.35
ax2.plot(Hc*h,np.rad2deg(th),'-',color=color[4])#'k-',alpha=0.5)
#ax2.plot(Hc*h,np.rad2deg(np.arcsin(h)),'-',color=color[4])
#ax2.errorbar(H,thetac,yerr=ethetac,fmt='s',color=color[4])
#Hc = 2
#ax2.plot(Hc*h,np.rad2deg(th),'k-',alpha=0.3)
#ax2.plot(Hc*h,np.rad2deg(np.arcsin(h)),'-',color=color[6])

ax2.text(2,40,r'$\alpha$ = 5.1(6) deg')
ax2.plot([1.8,1.9],[32,32],'-',color=color[4])
ax2.text(2,30,r'$H_c$ = 1.35 T')
#ax2.plot([1.8,1.9],[20,20],'-',color=color[6])
#ax2.text(2,20,r'$H_c$ = 2 T')
ax1.set_xlim(-0.1,3.5)
ax1.set_ylim(51,64.9)
ax2.set_xlim(-0.1,3.5)
ax2.set_ylim(-0.1,95)
ax2.set_ylabel(r'$\theta$ [deg]')
ax2.set_xlabel(r'$\mu_0$H [T]')
ax2.text(0,80,'(b)')
P.subplots_adjust(top=0.96,
bottom=0.11,
left=0.175,
right=0.94,
hspace=0.02)
P.show()
