import numpy as np
import matplotlib.pyplot as P
from iminuit import Minuit
from iminuit.util import make_func_code

class two: # definisce il modello come una classe con due metodi
    def __init__(self,x,y): # inizializzazione 
    #  carica la veriabile indipendente e i dati
        self.x = x
        self.y = y
    def ninegauss(self,x,A,nuQ,nuhf,s):
        # metodo base, calcola nove Gaussiane non normalizzata   
        y = np.zeros(x.shape)
        ds1,ds2 = 0.03,0.005    
        for k in range(-4,5):
            fk = 1#+ds2*k**2-k*ds1
            nu0 = 3*nuQ**2/(2*nuhf)     
            nuk = nu0*(1-k**2/8)+k*nuQ/2
            transition_element_squared = (25-k**2)/4 # (I(I+1)-m(m+1))/4, m=k+1/2 I=9/2, 99/4
            y += A*s*np.sqrt(2*np.pi)*transition_element_squared*np.exp(-0.5*((x-nuhf-nuk)/(s*fk))**2)                                     
        return y
#        ninegauss.func_code = make_func_code(["A","nuQ","nuhf","s"])
    def twosites(self,x,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2):
        # calcola due siti in proporzione 1:2
        return self.ninegauss(x,A,nuQ1,nuhf1,s1)+self.ninegauss(x,2*A,nuQ2,nuhf2,s2)
        twosites.func_code = make_func_code(["A","nuQ1","nuhf1","s1","nuQ2","nuhf2","s2"])
    def __call__(self,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2):
        # metodo utilizzato da minuit, calcola il chi quadro
        return sum((self.twosites(self.x,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2)-self.y)**2)
        
class four: # definisce il modello come una classe con due metodi
    def __init__(self,x,y): # inizializzazione 
    #  carica la veriabile indipendente e i dati
        self.x = x
        self.y = y
    def ninegauss(self,x,A,nuQ,nuhf,s):
        # metodo base, calcola nove Gaussiane non normalizzata   
        y = np.zeros(x.shape)
        ds1,ds2 = 0.03,0.005    
        for k in range(-4,5):
            fk = 1#+ds2*k**2-k*ds1
            nu0 = 3*nuQ**2/(2*nuhf)     
            nuk = nu0*(1-k**2/8)+k*nuQ/2
            transition_element_squared = (25-k**2)/4 # (I(I+1)-m(m+1))/4, m=k+1/2 I=9/2, 99/4
            y += A*s*np.sqrt(2*np.pi)*transition_element_squared*np.exp(-0.5*((x-nuhf-nuk)/(s*fk))**2)                                     
        return y
#        ninegauss.func_code = make_func_code(["A","nuQ","nuhf","s"])
    def twosites(self,x,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2):
        # calcola due siti in proporzione 1:2
        return self.ninegauss(x,A,nuQ1,nuhf1,s1)+self.ninegauss(x,2*A,nuQ2,nuhf2,s2)
#       twosites.func_code = make_func_code(["A","nuQ1","nuhf1","s1","nuQ2","nuhf2","s2"])
    def foursites(self,x,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b):
        return self.twosites(x,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2)+self.twosites(x,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b)
        foursites.func_code = make_func_code(["A","nuQ1","nuhf1","s1","nuQ2","nuhf2","s2","B","nuQ1b","nuhf1b","s1b","nuQ2b","nuhf2b","s2b"])
    def __call__(self,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b):
        # metodo utilizzato da minuit, calcola il chi quadro
        return sum((self.foursites(self.x,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b)-self.y)**2)
    

P.rcParams['font.size'] = '14'
prop_cycle = P.rcParams['axes.prop_cycle']
color = prop_cycle.by_key()['color']

p=np.genfromtxt('Nb93_ZFspectrum.dat')
nu = p[:,0]
a = p[:,1]-p[:,2]
offset = 1400

nbtwo = two(nu,a)
nbfour = four(nu,a)

A0,nuQ01,nuhf01,s0,nuQ02,nuhf02 = 1000., 0.89,8.9,0.1,1.24,13.8

fig,ax = P.subplots()
ax.plot(nu,a)
ax.plot(nu,nu-nu,'-k',lw=0.5)
ax.plot(nu,nu-nu-offset,'-k',lw=0.5)

Nbtwo = Minuit(nbtwo,A=A0,nuQ1=nuQ01,nuhf1=nuhf01,s1=s0,nuQ2=nuQ02,nuhf2=nuhf02,s2=s0)
Nbtwo.errordef = Minuit.LEAST_SQUARES
Nbtwo.migrad()
A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2 = Nbtwo.values
print('A = {:.0f}\nnuQ1= {:.2f} MHz,  nuhf1 = {:.2f} MHz,  s1 = {:.2f} MHz\nnuQ2= {:.2f} MHz,  nuhf2 = {:.2f} MHz,  s2 = {:.2f} MHz'.format(A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2))
ax.plot(nu,nbtwo.twosites(nu,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2),'--k',alpha=0.3)
ax.plot(nu,a-nbtwo.twosites(nu,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2)-offset,'--k',alpha=0.3)

Nb = Minuit(nbfour,A=A0*0.6,nuQ1=nuQ01,nuhf1=nuhf01,s1=s0,nuQ2=nuQ02,nuhf2=nuhf02,s2=s0,B=A0*0.05,nuQ1b=nuQ01*1.05,nuhf1b=nuhf01*1.05,s1b=0.3,nuQ2b=nuQ02*1.05,nuhf2b=nuhf02*1.05,s2b=0.3)
Nb.errordef = Minuit.LEAST_SQUARES
Nb.migrad()

A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b = Nb.values
print('A = {:.0f}\nnuQ1= {:.2f} MHz,  nuhf1 = {:.2f} MHz,  s1 = {:.2f} MHz\nnuQ2= {:.2f} MHz,  nuhf2 = {:.2f} MHz,  s2 = {:.2f} MHz'.format(A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2))
print('B = {:.0f}\nnuQ1b= {:.2f} MHz, nuhf1b = {:.2f} MHz, s1b = {:.2f} MHz\nnuQ2b= {:.2f} MHz, nuhf2b = {:.2f} MHz, s2b = {:.2f} MHz'.format(B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b))
ax.plot(nu,nbfour.foursites(nu,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b))
ax.plot(nu,a-nbfour.foursites(nu,A,nuQ1,nuhf1,s1,nuQ2,nuhf2,s2,B,nuQ1b,nuhf1b,s1b,nuQ2b,nuhf2b,s2b)-offset,color=color[1])

ax.set_xlabel('Frequency [MHz]')
ax.set_ylabel('NMR amplitude (arb. units)')
ax.set_xlim(6,20)
ax.set_ylim(-2*offset,6800)
ax.text(9,3200,r'$2a$')
ax.text(14.5,6300,r'$2b$')
P.subplots_adjust(top=0.95,
bottom=0.135,
left=0.180,
right=0.96)
P.show()
