import numpy as np
import matplotlib.pyplot as P
from scipy.io import loadmat

n = 1/np.sqrt(2*np.pi)
def fourgau(nu,a1,c1,s1,a2,c2,s2,a3,c3,s3,a4,c4,s4):
    a = a1*n/s1*np.exp(-0.5*((nu-c1)/s1)**2)
    a += a2*n/s2*np.exp(-0.5*((nu-c2)/s2)**2)
    a += a3*n/s3*np.exp(-0.5*((nu-c3)/s3)**2)
    a += a4*n/s4*np.exp(-0.5*((nu-c4)/s4)**2)
    return a
    
def gau(nu,a,c,s):
    return a*n/s*np.exp(-0.5*((nu-c)/s)**2)
    
P.rcParams['font.size'] = '14'
prop_cycle = P.rcParams['axes.prop_cycle']
color = prop_cycle.by_key()['color']

scala = 1000
p = np.genfromtxt('1000G.csv')
nu1000 = p[:,0]
A1000 = p[:,1]/1000


p = np.genfromtxt('GaussFit.dat',comments="#")
H = p[:,0]
a1  = p[:,1]/scala
c1 = p[:,2]
s1 = p[:,3]
a2  = p[:,4]/scala
c2 = p[:,5]
s2 = p[:,6]
a3  = p[:,7]/scala
c3 = p[:,8]
s3 = p[:,9]
a4  = p[:,10]/scala
c4 = p[:,11]
s4 = p[:,12]
p = loadmat('NbvsH.mat')
nu2 = p['nu2T'][0]
nu25 = p['nu25T'][0]
nu35 = p['nu35T'][0]
nu45 = p['nu45T'][0]
A2 = p['A2T'][0]/nu2**2/scala
A25 = p['A25T'][0]/nu25**2/scala
A35 = p['A35T'][0]/nu35**2/scala
A45 = p['A45T'][0]/nu45**2/scala

gam = 10.4523
one = np.array([1,1])
fig,ax = P.subplots(figsize=(5,5))
xmin,xmax  = np.array([17,22,30,40]),np.array([40,45,55,65])
da = 1.6
ms = 3
ax.plot([5,20.5],np.array([0,0])+da*0.1,'k-',lw=0.5,alpha=0.5)
ax.plot(nu1000,A1000,'o',color=color[4],mfc='w',ms=ms)
a = gau(nu1000,6,8.3+gam*0.1,1.25)+gau(nu1000,13,13.6+gam*0.1,1.5)+da*0.1
ax.plot(nu1000,a,'-',color=color[4])
ax.fill_between(nu1000,gau(nu1000,6,8.3+gam*0.1,1.25)+da*0.1,a-a+da*0.1,color=color[4],alpha=0.2)
ax.fill_between(nu1000,gau(nu1000,13,13.6+gam*0.1,1.5)+da*0.1,a-a+da*0.1,color=color[4],alpha=0.2)
ax.plot(8.3+gam*0.1,da*0.1,'o',color=color[4])
ax.plot(one*8.3+gam*0.1,np.array([0,6*n/1.25+da*0.1]),'-',color=color[4])
ax.plot(13.6+gam*0.1,da*0.1,'o',color=color[4])
ax.plot(one*13.6+gam*0.1,np.array([0,13.6*n/1.5+da*0.1]),'-',color=color[4])
ax.text(20,0.5,'H=0.1T')

k=0
a = fourgau(nu2,a1[k],c1[k],s1[k],a2[k],c2[k],s2[k],a3[k],c3[k],s3[k],a4[k],c4[k],s4[k])
ax.plot([xmin[k],xmax[k]],np.array([0,0])+da*2,'k-',lw=0.5,alpha=0.5)
ax.plot(nu2,A2+da*2,'o',color=color[0],mfc='w',ms=ms)
ax.plot(nu2,a+da*2,'-',color=color[0])
ax.fill_between(nu2,gau(nu2,a1[k],c1[k],s1[k])+da*2,nu2-nu2+da*2,color=color[0],alpha=0.2)
ax.fill_between(nu2,gau(nu2,a2[k],c2[k],s2[k])+da*2,nu2-nu2+da*2,color=color[0],alpha=0.2)
ax.fill_between(nu2,gau(nu2,a3[k],c3[k],s3[k])+da*2,nu2-nu2+da*2,color=color[0],alpha=0.2)
ax.fill_between(nu2,gau(nu2,a4[k],c4[k],s4[k])+da*2,nu2-nu2+da*2,color=color[0],alpha=0.2)
ax.plot(c2[k],da*2,'o',color=color[0])
ax.plot(one*c2[k],np.array([0,a2[k]*n/s2[k]])+da*2,'-',color=color[k])
ax.plot(c3[k],da*2,'o',color=color[0])
ax.plot(one*c3[k],np.array([0,a3[k]*n/s3[k]])+da*2,'-',color=color[k])
ax.text(33,2.7,'H=2T')

k=1
ax.plot([xmin[k],xmax[k]],np.array([0,0])+da*2.5,'k-',lw=0.5,alpha=0.5)
ax.plot(nu25[0:47],A25[0:47]+da*2.5,'o',color=color[1],mfc='w',ms=ms)
ax.plot(nu25[52:],A25[52:]+da*2.5,'o',color=color[1],mfc='w',ms=ms)
ax.text(39,3.5,'H=2.5T')


a = fourgau(nu25,a1[k],c1[k],s1[k],a2[k],c2[k],s2[k],a3[k],c3[k],s3[k],a4[k],c4[k],s4[k])
ax.plot(nu25,a+da*2.5,'-',color=color[1])
ax.fill_between(nu25,gau(nu25,a1[k],c1[k],s1[k])+da*2.5,nu25-nu25+da*2.5,color=color[1],alpha=0.2)
ax.fill_between(nu25,gau(nu25,a2[k],c2[k],s2[k])+da*2.5,nu25-nu25+da*2.5,color=color[1],alpha=0.2)
ax.fill_between(nu25,gau(nu25,a3[k],c3[k],s3[k])+da*2.5,nu25-nu25+da*2.5,color=color[1],alpha=0.2)
ax.fill_between(nu25,gau(nu25,a4[k],c4[k],s4[k])+da*2.5,nu25-nu25+da*2.5,color=color[1],alpha=0.2)
ax.plot(c1[k],2.5*da,'o',color=color[k])
ax.plot(one*c1[k],np.array([0,a1[k]*n/s1[k]])+2.5*da,'-',color=color[k])
ax.plot(c3[k],2.5*da,'o',color=color[k])
ax.plot(one*c3[k],np.array([0,a3[k]*n/s3[k]])+2.5*da,'-',color=color[k])
ax.plot(c4[k],2.5*da,'o',color=color[k])
ax.plot(one*c4[k],np.array([0,a4[k]*n/s4[k]])+2.5*da,'-',color=color[k])

k=2
ax.plot([xmin[k],xmax[k]],np.array([0,0])+da*3.5,'k-',lw=0.5,alpha=0.5)
ax.plot(nu35,A35+da*3.5,'o',color=color[2],mfc='w',ms=ms)
a = fourgau(nu35,a1[k],c1[k],s1[k],a2[k],c2[k],s2[k],a3[k],c3[k],s3[k],a4[k],c4[k],s4[k])
ax.plot(nu35,a+da*3.5,'-',color=color[2])
ax.fill_between(nu35,gau(nu35,a1[k],c1[k],s1[k])+da*3.5,nu35-nu35+da*3.5,color=color[2],alpha=0.2)
ax.fill_between(nu35,gau(nu35,a2[k],c2[k],s2[k])+da*3.5,nu35-nu35+da*3.5,color=color[2],alpha=0.2)
ax.fill_between(nu35,gau(nu35,a3[k],c3[k],s3[k])+da*3.5,nu35-nu35+da*3.5,color=color[2],alpha=0.2)
ax.fill_between(nu35,gau(nu35,a4[k],c4[k],s4[k])+da*3.5,nu35-nu35+da*3.5,color=color[2],alpha=0.2)
ax.plot(c1[k],3.5*da,'o',color=color[k])
ax.plot(one*c1[k],np.array([0,a1[k]*n/s1[k]])+3.5*da,'-',color=color[k])
ax.plot(c3[k],3.5*da,'o',color=color[k])
ax.plot(one*c3[k],np.array([0,a3[k]*n/s3[k]])+3.5*da,'-',color=color[k])
ax.plot(c4[k],3.5*da,'o',color=color[k])
ax.plot(one*c4[k],np.array([0,a4[k]*n/s4[k]])+3.5*da,'-',color=color[k])
ax.text(50,5.2,'H=3.5T')

ax.text(29,4.8,'0')
ax.text(18.3,2.2,r'$2a$')
ax.text(26,1.7,r'$2b$')
k=3
ax.plot([xmin[k],xmax[k]],np.array([0,0])+da*4.5,'k-',lw=0.5,alpha=0.5)
ax.plot(nu45,A45+da*4.5,'o',color=color[3],mfc='w',ms=ms)
a = fourgau(nu45,a1[k],c1[k],s1[k],a2[k],c2[k],s2[k],a3[k],c3[k],s3[k],a4[k],c4[k],s4[k])
ax.plot(nu45,a+da*4.5,'-',color=color[3])
ax.fill_between(nu45,gau(nu45,a1[k],c1[k],s1[k])+da*4.5,nu45-nu45+da*4.5,color=color[3],alpha=0.2)
ax.fill_between(nu45,gau(nu45,a2[k],c2[k],s2[k])+da*4.5,nu45-nu45+da*4.5,color=color[3],alpha=0.2)
ax.fill_between(nu45,gau(nu45,a3[k],c3[k],s3[k])+da*4.5,nu45-nu45+da*4.5,color=color[3],alpha=0.2)
ax.fill_between(nu45,gau(nu45,a4[k],c4[k],s4[k])+da*4.5,nu45-nu45+da*4.5,color=color[3],alpha=0.2)
ax.plot(c1[k],4.5*da,'o',color=color[k])
ax.plot(one*c1[k],np.array([0,a1[k]*n/s1[k]])+4.5*da,'-',color=color[k])
ax.plot(c3[k],4.5*da,'o',color=color[k])
ax.plot(one*c3[k],np.array([0,a3[k]*n/s3[k]])+4.5*da,'-',color=color[k])
ax.plot(c4[k],4.5*da,'o',color=color[k])
ax.plot(one*c4[k],np.array([0,a4[k]*n/s4[k]])+4.5*da,'-',color=color[k])
ax.text(55,7.7,'H=4.5T')

ax.set_xlabel(r'$\nu$ [MHz]')
ax.set_ylabel('NMR amplitude (arb. units)')
ax.set_xlim(0,67)
ax.set_ylim([0,8.3])
P.subplots_adjust(top=0.955,
bottom=0.13,
left=0.11,
right=0.96)

H01,H02,H03 = 0,0.75, 1.2
H = np.array([0.1,2,2.5,3.5,4.5])
y = da*H
ax.plot(gam*(H+H01),y,'k--',alpha=0.5)
ax.plot(gam*(H+H02),y,'k--',alpha=0.5)
ax.plot(gam*(H+H03),y,'k--',alpha=0.5)

P.show()
