import numpy as np
import matplotlib.pyplot as P
from iminuit import Minuit
from iminuit.util import make_func_code


class fcn: # definisce il modello come una classe con due metodi
    def __init__(self,x,y): # inizializzazione 
    #  carica la veriabile indipendente e i dati
        self.x = x
        self.y = y
    def muba(self,x,N,T2,a,nu,phi,tau,B):
        t = x-2e-6
        return N*np.exp(-t/T2)*(1+a*np.cos(2*np.pi*(nu*t+phi/360))*np.exp(-t/tau))+B
        muba.func_code = make_func_code(["N","T2","a","nu","phi","tau","B"])
    def __call__(self,N,T2,a,nu,phi,tau,B):
        # metodo utilizzato da minuit, calcola il chi quadro
        return sum(((self.muba(self.x,N,T2,a,nu,phi,tau,B)-self.y))**2)

P.rcParams['font.size'] = '14'
lw=2.5
ms=6
mew=2.5
fig,[[ax6,ax13],[ax15,ax20]] = P.subplots(2,2,figsize=(7,6),sharex=True,sharey=True)

filename = '/home/roberto.derenzi/tex/drafts/CrNb3S6-MnNb3S6/Overleaf/MaterialsCloud/tmp/t2_0.6T.txt'
p = np.genfromtxt(filename,skip_header=1)
scale = 1
ax6.plot(p[:,0],scale*p[:,1],'ro',ms=ms,mfc='w',mew=mew)

model = fcn(p[:,0],p[:,1])
bestfit = Minuit(model,N=5e4,T2=1e-5,a=0.2,nu=2e6,phi=10,tau=3e-5,B=5e3)
bestfit.errordef = Minuit.LEAST_SQUARES
print(bestfit.migrad())
N,T2,a,nu,phi,tau,B = bestfit.values
x = np.linspace(2e-6,5.4e-6,100)
ax6.plot(x,scale*model.muba(x,N,T2,a,nu,phi,tau,B),lw=lw)
ax6.ticklabel_format(axis='y', scilimits=[-3, 3])
ax6.text(4.5e-6,5e4,'0.6 T')
#ax6.set_ylabel('Spin echo Amplitude [Arb. units]')

filename = '/home/roberto.derenzi/tex/drafts/CrNb3S6-MnNb3S6/Overleaf/MaterialsCloud/tmp/t2_1.3T.txt'
p = np.genfromtxt(filename,skip_header=1)
scale = 5/3.5
ax13.plot(p[:,0],scale*p[:,1],'ro',ms=ms,mfc='w',mew=mew)
model = fcn(p[:,0],p[:,1])
bestfit = Minuit(model,N=5e4,T2=1e-5,a=0.2,nu=2e6,phi=10,tau=3e-5,B=5e3)
bestfit.errordef = Minuit.LEAST_SQUARES
print(bestfit.migrad())
N,T2,a,nu,phi,tau,B = bestfit.values
ax13.plot(x,scale*model.muba(x,N,T2,a,nu,phi,tau,B),lw=lw)
ax13.text(4.5e-6,5e4,'1.3 T')

filename = '/home/roberto.derenzi/tex/drafts/CrNb3S6-MnNb3S6/Overleaf/MaterialsCloud/tmp/FigS.1.1.5T.dat'
p = np.genfromtxt(filename,skip_header=1)
scale = 5/3.5
ax15.plot(p[:,0],scale*p[:,1],'ro',ms=ms,mfc='w',mew=mew)
model = fcn(p[:,0],p[:,1])
bestfit = Minuit(model,N=5e4,T2=1e-5,a=0.2,nu=2e6,phi=10,tau=3e-5,B=5e3)
bestfit.errordef = Minuit.LEAST_SQUARES
print(bestfit.migrad())
N,T2,a,nu,phi,tau,B = bestfit.values
ax15.plot(x,scale*model.muba(x,N,T2,a,nu,phi,tau,B),lw=lw)
ax15.text(4.5e-6,5e4,'1.5 T')
#ax15.set_xlabel(r'Delay $\tau$ [s]')
#ax15.set_ylabel('Spin echo Amplitude [Arb. units]')

filename = '/home/roberto.derenzi/tex/drafts/CrNb3S6-MnNb3S6/Overleaf/MaterialsCloud/tmp/FigS.1.2.0T.dat'
p = np.genfromtxt(filename,skip_header=1)
scale = 5/12
ax20.plot(p[:,0],scale*p[:,1],'ro',ms=ms,mfc='w',mew=mew)
model = fcn(p[:,0],p[:,1])
bestfit = Minuit(model,N=5e4,T2=1e-5,a=0,nu=0,phi=0,tau=0,B=5e3)
bestfit.fixed["a"] = True
bestfit.fixed["nu"] = True
bestfit.fixed["phi"] = True
bestfit.fixed["tau"] = True
bestfit.errordef = Minuit.LEAST_SQUARES
print(bestfit.migrad())
N,T2,a,nu,phi,tau,B = bestfit.values
ax20.plot(x,scale*model.muba(x,N,T2,a,nu,phi,tau,B),lw=lw)
ax20.text(4.5e-6,5e4,'2.0 T')
ax20.set_xlim(2e-6,5.4e-6)
fig.supylabel('Spin echo Amplitude [Arb. units]')
fig.supxlabel(r'Delay $2\tau$ [s]')
#ax20.set_xlabel(r'Delay $\tau$ [s]')
P.subplots_adjust(top=0.95,
bottom=0.17,
left=0.11,
right=0.965,
hspace=0.085,
wspace=0.02)
P.show()
